//
//  AppDelegate.swift
//  AsReaderGUNSample
//
//  Created by mac on 2018/5/24.
//  Copyright © 2018年 Asterisk. All rights reserved.
//

import UIKit
import AsReaderGunSDK
@UIApplicationMain
final class AppDelegate: UIResponder, UIApplicationDelegate {
    var systemSetting: SystemSetting?
    var window: UIWindow?
    var mAsReaderGUN: AsReaderGUN?
    var mReader: AsReader?
    var hud: MBProgressHUD?
    var insertOperation: OperationQueue?
    var backupUseMask: SelectFlag?
    var backupSelectMasks: [AsSelectMaskParam] = []
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplicationLaunchOptionsKey: Any]?) -> Bool {
        insertOperation = OperationQueue()
        systemSetting = SystemSetting.shared()
        if let result = systemSetting?.getNotFirstUsed() {
            if !result {
                systemSetting?.setNotFirstUsed(true)
                systemSetting?.setBuzzer(Int(Buzzer_High.rawValue))
                systemSetting?.setVibrator(Int(Vibrator_On.rawValue))
                systemSetting?.setInventoryTime(400)
                systemSetting?.setIdleTime(0)
                systemSetting?.setAutoOffTime(1800)
                systemSetting?.setQValue(4)
                systemSetting?.setAlgorithm(Int(DynamicQ.rawValue))
                systemSetting?.setMaxQ(15)
                systemSetting?.setMinQ(0)
                systemSetting?.setSleepTime(300)
                systemSetting?.setMaskTypeValue(Int(MaskType_NO_MASK.rawValue))
                systemSetting?.setLinkProfileValue(1)
                systemSetting?.setDefaultLinkProfileValue(1)
                systemSetting?.setOperationTime(0)
                systemSetting?.setPowerGain(300)
                systemSetting?.setInventorySession(Int(Session_S0.rawValue))
                systemSetting?.setSessionFlag(Int(SessionFlag_AB.rawValue))
                systemSetting?.setContinuousMode(true)
                systemSetting?.setRssiMode(false)
                systemSetting?.setPropBarcodeMode(true)
                systemSetting?.setEpcMaskMatchMode(false)
                systemSetting?.setDisPlayPC(true)
                systemSetting?.setLbtItemData(nil)
                systemSetting?.setEpcMaskData(nil)
                systemSetting?.setUseSelectionMask(Int(SelectFlag_All.rawValue))
                systemSetting?.setSelectionEpcMaskData(nil)
                systemSetting?.setBarcodeParamData(nil)
                systemSetting?.setBuzzerTime(1)
                systemSetting?.setVibratorTime(1)
                systemSetting?.setBarcodeReScanTime(0)
            }
        }
        redirectConsoleLog(toDocumentFolder: "log.txt")
        AsRingDeviceSDK.shared()
        removeLogFile()
        redirectNSLogToDocumentFolder()
        return true
    }
    func applicationWillResignActive(_ application: UIApplication) {
    }
    func applicationDidEnterBackground(_ application: UIApplication) {
    }
    func applicationWillEnterForeground(_ application: UIApplication) {
    }
    func applicationDidBecomeActive(_ application: UIApplication) {
    }
    func applicationWillTerminate(_ application: UIApplication) {
    }
    // MARK: - Blaster Method
    func initMask(_ maskValue: String?, maskType: MaskType) {
        guard let nowMaskValue = maskValue else {
            return
        }
        guard !nowMaskValue.isEmpty else {
            return
        }
        if maskType == MaskType_Selection {
            backupUseMask = mReader?.useSelectionMask
            for index in MIN_SELECTION_MASK..<MAX_SELECTION_MASK {
                if let result = mReader?.usedSelectionMask(index) {
                    if !result {
                        break
                    }
                }
                let param: AsSelectMaskParam? = mReader?.getSelectionMask(index)
                if let aParam = param {
                    backupSelectMasks.append(aParam)
                }
            }
        }
        let param = AsSelectMaskParam(parameterIndex: 0, target: MaskTarget_SL, action: MaskAction_AB, bank: Bank_EPC, offset: 32, mask: (nowMaskValue as NSString?)?.substring(from: 4), used: true)
        mReader?.setSelectionMask(0, with: param)
        mReader?.useSelectionMask = SelectFlag_SL
        for index in 1..<MAX_SELECTION_MASK {
            mReader?.setMaskUsed(index, used: false)
        }
    }
    func exitMask() {
        if let aBackupUseMask = backupUseMask {
            mReader?.useSelectionMask = aBackupUseMask
        }
        for index in MIN_SELECTION_MASK..<MAX_SELECTION_MASK {
            print("count: \(backupSelectMasks.count)")
            if backupSelectMasks.count < index + 1 {
                mReader?.removeSelectionMask(index)
                break
            }
            let param = backupSelectMasks[Int(index)]
            mReader?.setSelectionMask(index, with: param)
        }
        backupSelectMasks.removeAll()
    }
    func getAppVersion() -> String? {
        let infoDictionary = Bundle.main.infoDictionary
        let majorVersion = infoDictionary?["CFBundleShortVersionString"] as? String
        let minorVersion = infoDictionary?["CFBundleVersion"] as? String
        return "Version \(majorVersion ?? "") Build \(minorVersion ?? "")"
    }
    func getCurrentVC() -> UIViewController? {
        var result: UIViewController?
        var window: UIWindow? = UIApplication.shared.keyWindow
        if window?.windowLevel != UIWindowLevelNormal {
            let windows = UIApplication.shared.windows
            for tmpWin: UIWindow in windows  where tmpWin.windowLevel == UIWindowLevelNormal {
                window = tmpWin
                break
            }
        }
        let frontView: UIView? = window?.subviews[0]
        let nextResponder = frontView?.next
        if nextResponder is UIViewController {
            result = nextResponder as? UIViewController
        } else {
            result = window?.rootViewController
        }
        return result
    }
    final class func showGlobalProgressHUD(withTitle title: String?) {
        var result: UIViewController?
        var window: UIWindow? = UIApplication.shared.keyWindow
        if window?.windowLevel != UIWindowLevelNormal {
            let windows = UIApplication.shared.windows
            for tmpWin: UIWindow in windows where tmpWin.windowLevel == UIWindowLevelNormal {
                window = tmpWin
                break
            }
        }
        let frontView: UIView? = window?.subviews[0]
        let nextResponder = frontView?.next
        if nextResponder is UIViewController {
            result = nextResponder as? UIViewController
        } else {
            result = window?.rootViewController
        }
        guard let nowResult = result else {
            return
        }
        guard let view = nowResult.view  else {
            return
        }
        guard let nowTitle = title else {
            return
        }
        let hud = MBProgressHUD.showAdded(to: view, animated: true)
        hud.label.text = nowTitle
    }

    final class func dismissGlobalHUD() {
        var result: UIViewController?
        var window: UIWindow? = UIApplication.shared.keyWindow
        if window?.windowLevel != UIWindowLevelNormal {
            let windows = UIApplication.shared.windows
            for tmpWin: UIWindow in windows  where tmpWin.windowLevel == UIWindowLevelNormal {
                window = tmpWin
                break
            }
        }
        let frontView: UIView? = window?.subviews[0]
        let nextResponder = frontView?.next
        if nextResponder is UIViewController {
            result = nextResponder as? UIViewController
        } else {
            result = window?.rootViewController
        }
        guard let nowResult = result else {
            return
        }
        guard let view = nowResult.view  else {
            return
        }
        MBProgressHUD.hide(for: view, animated: true)
    }

    func redirectNSLogToDocumentFolder() {
        let paths = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)
        let documentsDirectory = paths[0]
        let fileName = "writelog.txt"
        let logFilePath = URL(fileURLWithPath: documentsDirectory).appendingPathComponent(fileName).absoluteString
        freopen(logFilePath.cString(using: String.Encoding.ascii), "a+", stderr)
    }
    func removeLogFile() {
        let paths = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)
        let documentsDirectory = paths[0]
        let fileName = "writelog.txt"
        let logFilePath = URL(fileURLWithPath: documentsDirectory).appendingPathComponent(fileName).absoluteString
        var logSize: Float = 0
        let manager = FileManager.default
        if manager.fileExists(atPath: logFilePath) {
            do {
                let attr: NSDictionary = try manager.attributesOfItem(atPath: logFilePath) as NSDictionary
                logSize = Float(attr.fileSize())
            } catch {
            }
        }
        if logSize >= 1 {
            if FileManager.default.fileExists(atPath: logFilePath) {
                try? FileManager.default.removeItem(atPath: logFilePath)
            }
        }
    }
    func redirectConsoleLog(toDocumentFolder filename: String?) {
        let paths = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)
        let documentsDirectory = paths[0]
        let logPath = URL(fileURLWithPath: documentsDirectory).appendingPathComponent(filename ?? "").absoluteString
        freopen(logPath.cString(using: String.Encoding.ascii), "w", stderr)
    }
}
