//
//  BarcodeOptionViewController.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/12.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit

final class BarcodeOptionViewController: UIViewController {
    @IBOutlet private weak var tableView: UITableView!
    private weak var appDelegate: AppDelegate?
    private var systemSetting: SystemSetting?
    private var barcodeParamData: [AsParamValue] = []
    override func viewDidLoad() {
        super.viewDidLoad()
        AppDelegate.showGlobalProgressHUD(withTitle: loading)
        appDelegate = UIApplication.shared.delegate as? AppDelegate
        appDelegate?.mReader?.setDelegate(self)
        systemSetting = SystemSetting.shared()
        if appDelegate?.mReader != nil {
            if let isRFIDModule = appDelegate?.mReader?.isRFIDModule() {
                if !isRFIDModule {
                    let rightItem = UIBarButtonItem(title: "Setting",
                                                    style: .done,
                                                    target: self,
                                                    action: #selector(setting))
                    navigationItem.rightBarButtonItem = rightItem
                }
            }
         } else {
            return
        }
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(asReaderGUNDisconnected(_:)),
                                               name: NSNotification.Name(asreaderGUNDisconnected),
                                               object: nil)
        appDelegate?.mReader?.setDelegate(self)
        appDelegate?.mReader?.setScanMode(BarcodeScanMode)
    }
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(true)
        appDelegate?.mReader?.setPropBarcodeMode(true)
        let paramData: [ParamName] = [UPCA, UPCE0, UPCE1, EAN8,
                                      EAN13, Code128, Code39, Code39Pharmaceutical,
                                      Code93, Code11, I2of5, Codabar,
                                      MSI, X2of5, PDF417, MacroPDF417,
                                      MicroPDF, Matrix, MaxiCode, QRCode,
                                      AztecCode, NEC2of5, R2of5, A2of5,
                                      GS1128, Telepen, RSS14, RSSLimit,
                                      RSSExp, TriopticCode, CodablockA, CodablockF,
                                      ComCode, TLC39, ChinaPost, KoreaPost, HanXinCode]
        var arrParam: [AsParamValue] = []
        for paramName in paramData {
            let paramNameNum: NSNumber = NSNumber.init(value: Int64(paramName.rawValue))
            if let val = appDelegate?.mReader?.getBarcodeParam(paramNameNum) {
                    arrParam.append(val)
            }
        }
        if let barcodePar =  systemSetting?.getBarcodeParamData() {
            if barcodePar.isEmpty {
                barcodeParamData = barcodePar
            } else {
                barcodeParamData = arrParam
            }
          } else {
            barcodeParamData = arrParam
        }
        tableView.reloadData()
        AppDelegate.dismissGlobalHUD()
    }
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name(asreaderGUNDisconnected),
                                                  object: nil)
        appDelegate?.mReader?.setScanMode(RFIDScanMode)
        appDelegate?.mReader?.setBarcodeMode(false, isKeyAction: false)
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    @IBAction private func defaultAllSymBtnTapped(_ sender: UIButton) {
        for paramValue in barcodeParamData {
            switch paramValue.paramName {
            case UPCA:
                paramValue.setEnabled(true)
            case UPCE0:
                paramValue.setEnabled(true)
            case UPCE1:
                paramValue.setEnabled(false)
            case EAN8:
                paramValue.setEnabled(true)
            case EAN13:
                paramValue.setEnabled(true)
            case Code128:
                paramValue.setEnabled(true)
            case Code39:
                paramValue.setEnabled(true)
            case Code39Pharmaceutical:
                paramValue.setEnabled(false)
            case Code93:
                paramValue.setEnabled(true)
            case Code11:
                paramValue.setEnabled(false)
            case I2of5:
                paramValue.setEnabled(true)
            case Codabar:
                paramValue.setEnabled(true)
            case MSI:
                paramValue.setEnabled(false)
            case X2of5:
                paramValue.setEnabled(false)
            case PDF417:
                paramValue.setEnabled(true)
            case MacroPDF417:
                paramValue.setEnabled(true)
            case MicroPDF:
                paramValue.setEnabled(false)
            case Matrix:
                paramValue.setEnabled(false)
            case MaxiCode:
                paramValue.setEnabled(true)
            case QRCode:
                paramValue.setEnabled(true)
            case AztecCode:
                paramValue.setEnabled(true)
            case NEC2of5:
                paramValue.setEnabled(true)
            case R2of5:
                paramValue.setEnabled(false)
            case A2of5:
                paramValue.setEnabled(false)
            case GS1128:
                paramValue.setEnabled(true)
            case Telepen:
                paramValue.setEnabled(false)
            case RSS14:
                paramValue.setEnabled(true)
            case RSSLimit:
                paramValue.setEnabled(false)
            case RSSExp:
                paramValue.setEnabled(false)
            case TriopticCode:
                paramValue.setEnabled(false)
            case CodablockA:
                paramValue.setEnabled(false)
            case CodablockF:
                paramValue.setEnabled(false)
            case ComCode:
                paramValue.setEnabled(true)
            case TLC39:
                paramValue.setEnabled(false)
            case ChinaPost:
                paramValue.setEnabled(false)
            case KoreaPost:
                paramValue.setEnabled(false)
            case HanXinCode:
                paramValue.setEnabled(false)
            default:
                break
            }
        }
        tableView.reloadData()
    }
    @IBAction private func disableAllSymBtnTapped(_ sender: UIButton) {
        for paramValue in barcodeParamData {
            paramValue.setEnabled(false)
        }
        tableView.reloadData()
    }
    @IBAction private func enableAllSymBtnTapped(_ sender: UIButton) {
        for paramValue in barcodeParamData {
            paramValue.setEnabled(true)
        }
        tableView.reloadData()
    }
    @IBAction private func saveSymBtnTapped(_ sender: UIButton) {
        var res = ResultNoError
        AppDelegate.showGlobalProgressHUD(withTitle: "Saving...")
        for paramName in barcodeParamData {
            if let resultCode = appDelegate?.mReader?.setBarcodeParam(paramName) {
                res = resultCode
                if res != ResultNoError {
                    AppDelegate.dismissGlobalHUD()
                    return
                }
            }
        }
        AppDelegate.dismissGlobalHUD()
        if res == ResultNoError {
            let alertController = UIAlertController(title: "Success",
                                                    message: "Saved successfully",
                                                    preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
        } else {
            let alertController = UIAlertController(title: "Error",
                                                    message: "Failed to save",
                                                    preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
        }
        systemSetting?.setBarcodeParamData(barcodeParamData)
    }
}
// MARK: - Custom
extension BarcodeOptionViewController {
    @objc private func setting() {
        let barcodeViewController = storyboard?.instantiateViewController(withIdentifier: "BarcodeOptionVC") as? BarcodeOptionVC
        if let aController = barcodeViewController {
            navigationController?.pushViewController(aController, animated: true)
        }
    }
    @objc private func asReaderGUNDisconnected(_ notification: Notification?) {
        DispatchQueue.main.async {
            self.appDelegate?.mReader = nil
            AppDelegate.dismissGlobalHUD()
            self.navigationController?.popToRootViewController(animated: true)
        }
    }
}
// MARK: - UITableViewDelegate, UITableViewDataSource
extension BarcodeOptionViewController: UITableViewDelegate, UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return barcodeParamData.count
    }
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: UITableViewCell? = tableView.dequeueReusableCell(withIdentifier: "BarcodeCell", for: indexPath)
        let paramValue = barcodeParamData[indexPath.row]
        cell?.textLabel?.text = AsParamName.getName(paramValue.paramName)
        if paramValue.value != 0 {
            cell?.accessoryType = .checkmark
        } else {
            cell?.accessoryType = .none
        }
        guard let aCell = cell else {
            return UITableViewCell()
        }
        return aCell
    }
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let cell: UITableViewCell? = tableView.cellForRow(at: indexPath)
        let paramValue = barcodeParamData[indexPath.row]
        if cell?.accessoryType == .checkmark {
            cell?.accessoryType = .none
            paramValue.value = 0
        } else {
            cell?.accessoryType = .checkmark
            paramValue.value = 1
        }
    }
}
// MARK: - AsReaderDelegate
extension BarcodeOptionViewController: AsReaderDelegate {
    func on(asReaderTriggerKeyEvent status: Bool) -> Bool {
        return false
    }
    func on(asReaderLeftModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func on(asReaderRightModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func updateDeviceState(_ error: ResultCode) {
        if error != ResultNoError {
            let alertController = UIAlertController(title: "Error",
                                                    message: "Failed to get response. Try again.",
                                                    preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
                self.navigationController?.popViewController(animated: true)
            })
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
        }
    }
    func readTag(_ tag: String?, rssi: Float, phase: Float) {
    }
    func changedActionState(_ action: CommandType, resultCode: Int) {
    }
    func accessResult(_ error: ResultCode, actionState action: CommandType, epc: String?, data: String?, rssi: Float, phase: Float, frequency: Float) {
    }
}
