//
//  BarcodeDemoViewController.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/11.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit

final class BarcodeDemoViewController: UIViewController {
    @IBOutlet private weak var tableView: UITableView!
    @IBOutlet private weak var outputBtn: UIBarButtonItem!
    @IBOutlet private weak var pickerView: UIPickerView!
    @IBOutlet private weak var rescanTimePickerTopView: UIView!
    @IBOutlet private weak var rescanTimeTextField: UITextField!
    @IBOutlet private weak var startScanBtn: UIButton!
    @IBOutlet private weak var totalCountLabel: UILabel!
    @IBOutlet private var toDisableElems: [UIControl]!
    private weak var appDelegate: AppDelegate?
    private var isScanning: Bool = false
    private var rescanTimeValue: Int = 0
    private var barcodeData: [AsBarcodeItem] = []
    private lazy var rescanTimePickerData: [String] = {
        return rescanTimePickerArr
    }()
    private lazy var rescanTimeData: [String] = {
        return rescanTimeArr
    }()
    override func viewDidLoad() {
        super.viewDidLoad()
        if let  systemSetting = SystemSetting.shared() {
            rescanTimeTextField.text = rescanTimePickerData[Int(systemSetting.getBarcodeReScanTime())]
            rescanTimeValue = Int(systemSetting.getBarcodeReScanTime())
        }
        if let barcodeArr =  SystemSetting.shared()?.getBarcodeData() {
             barcodeData = barcodeArr
          } else {
            barcodeData = []
        }
        totalCountLabel.text = "\(barcodeData.count)"
        AppDelegate.showGlobalProgressHUD(withTitle: loading)
        appDelegate = UIApplication.shared.delegate as? AppDelegate
        appDelegate?.mReader?.setDelegate(self)
        if appDelegate?.mReader == nil {
            return
        }
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(asReaderGUNDisconnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNDisconnected),
                                               object: nil)
        appDelegate?.mReader?.setDelegate(self)
    }
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        AppDelegate.dismissGlobalHUD()
        appDelegate?.mReader?.setBarcodeMode(true, isKeyAction: true)
        appDelegate?.mReader?.setScanMode(BarcodeScanMode)
    }
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name(asreaderGUNDisconnected),
                                                  object: nil)
        appDelegate?.mReader?.setScanMode(RFIDScanMode)
        appDelegate?.mReader?.setBarcodeMode(false, isKeyAction: false)
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    @IBAction private func clearBtnTapped(_ sender: UIButton) {
        barcodeData.removeAll()
        SystemSetting.shared()?.setBarcodeData(barcodeData)
        tableView.reloadData()
        totalCountLabel.text = "\(barcodeData.count)"
    }
    @IBAction private func scanStartBtnTapped(_ sender: UIButton) {
        if isScanning != true {
            scanEnabled(true)
        } else {
            scanEnabled(false)
        }
    }
    @IBAction private func rescanTimeBtnTapped(_ sender: UIButton) {
        pickerView.isHidden = false
        pickerView.selectRow(rescanTimeValue, inComponent: 0, animated: false)
        rescanTimePickerTopView.isHidden = false
    }
    @IBAction private func pickerCloseBtnTapped(_ sender: UIButton) {
        hidePickerView()
    }
    @IBAction private func rescanTimePickerOkBtnTapped(_ sender: UIButton) {
        let idx: Int = pickerView.selectedRow(inComponent: 0)
        rescanTimeTextField.text = rescanTimePickerData[idx]
        rescanTimeValue = idx
        SystemSetting.shared()?.setBarcodeReScanTime(Int(rescanTimeValue))
        hidePickerView()
    }
    @IBAction private func outputCSVBtnTapped(_ sender: UIBarButtonItem) {
        DispatchQueue.main.async {
            var rfidCount: Int = 0
            for index in 0..<self.barcodeData.count {
                let item: AsBarcodeItem? = self.barcodeData[index]
                if item?.barcode != nil {
                    rfidCount += 1
                }
            }
            let fileName = "barcode.csv"
            if rfidCount != 0 {
                if CSVData.isFileExist(fileName) {
                    CSVData.deleteFile(forFileName: fileName)
                }
                CSVData.createCSV(withFileName: fileName)
                CSVData.exportBarcodeCSV(fileName, withArray: self.barcodeData)
            } else {
                return
            }
        }
    }
}
// MARK: - Custom
extension BarcodeDemoViewController {
    private func hidePickerView() {
        pickerView.isHidden = true
        rescanTimePickerTopView.isHidden = true
    }
    private func scanEnabled(_ enabled: Bool) {
        if enabled {
            appDelegate?.mReader?.startDecode()
            scanElemEnabled(false)
        } else {
            appDelegate?.mReader?.stopSync()
            scanElemEnabled(true)
        }
    }
    private func scanElemEnabled(_ enabled: Bool) {
        if enabled {
            isScanning = false
            startScanBtn.setTitle("Start Scan", for: .normal)
            for control in toDisableElems {
                control.isEnabled = true
            }
            outputBtn.isEnabled = true
        } else {
            isScanning = true
            startScanBtn.setTitle("Stop Scan", for: .normal)
            for control in toDisableElems {
                control.isEnabled = false
            }
            outputBtn.isEnabled = false
        }
    }
}
// MARK: - UITableViewDelegate, UITableViewDataSource
extension BarcodeDemoViewController: UITableViewDelegate, UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return barcodeData.count
    }
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "BarcodeCell",
                                                 for: indexPath) as? BarcodeDemoTableViewCell
        let item = barcodeData[indexPath.row]
        cell?.setCodeTypeValue(AsBarcodeType.getString(BarcodeType(rawValue: item.barcodeType)))
        cell?.setCodeIdValue(item.codeId)
        cell?.setBarcodeValue(item.barcode)
        guard let aCell = cell else {
            return UITableViewCell()
        }
         return aCell
    }
    func tableView(_ tableView: UITableView, editingStyleForRowAt indexPath: IndexPath) -> UITableViewCellEditingStyle {
        return .delete
    }
}
// MARK: - UIPickerViewDelegate,UIPickerViewDataSource
extension BarcodeDemoViewController: UIPickerViewDelegate, UIPickerViewDataSource {
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return rescanTimePickerData.count
    }
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        return rescanTimePickerData[row]
    }
}
// MARK: - AsReaderDelegate
extension BarcodeDemoViewController: AsReaderDelegate {
    func accessResult(_ error: ResultCode, actionState action: CommandType, epc: String?, data: String?, rssi: Float, phase: Float, frequency: Float) {
    }
    func readTag(_ tag: String?, rssi: Float, phase: Float) {
    }
    func detect(_ barcodeType: BarcodeType, codeId: String!, barcode: String!) {
        if barcodeType != BarcodeTypeNoRead {
            let timeDate = Date()
            let formatter = DateFormatter()
            formatter.dateStyle = .medium
            formatter.timeStyle = .short
            formatter.dateFormat = "YYYY/MM/dd HH:mm:ss"
            let dateTime = formatter.string(from: timeDate)
            let item = AsBarcodeItem(barcodeType: barcodeType, codeId: codeId, barcode: barcode, dateTime: dateTime)
            barcodeData.append(item)
            SystemSetting.shared()?.setBarcodeData(barcodeData)
            
             DispatchQueue.main.async {
                self.tableView.reloadData()
                self.tableView.scrollToRow(at: IndexPath(row: self.barcodeData.count - 1, section: 0), at: .bottom, animated: true)
                self.totalCountLabel.text = "\(self.barcodeData.count)"
            }
            
        }
    }
    func changedActionState(_ action: CommandType, resultCode: Int) {
        if action == CommandStop {
            if let rescanTime = Float(rescanTimeData[rescanTimeValue]) {
                if rescanTime > 0 && isScanning {
                    let timeInterval: TimeInterval = TimeInterval(rescanTime/1000.00)
                    DispatchQueue.main.asyncAfter(deadline: .now() + timeInterval, execute: {
                        self.tableView?.reloadData()
                        if self.isScanning {
                            self.appDelegate?.mReader?.startDecode()
                        }
                    })
                } else {
                    scanElemEnabled(true)
                }
            }
        }
    }
    func updateDeviceState(_ error: ResultCode) {
        if error != ResultNoError {
            let alertController = UIAlertController(title: "Error",
                                                    message: "Failed to get response. Try again.",
                                                    preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
                self.navigationController?.popViewController(animated: true)
            })
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
        }
    }
    func on(asReaderTriggerKeyEvent status: Bool) -> Bool {
        isScanning = status
        if isScanning {
            scanElemEnabled(false)
        }
        return true
    }
    func on(asReaderLeftModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func on(asReaderRightModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
}
// MARK: - NSNotification
extension BarcodeDemoViewController {
    @objc func asReaderGUNDisconnected(_ notification: Notification?) {
        DispatchQueue.main.async {
            self.appDelegate?.mReader = nil
            self.navigationController?.popToRootViewController(animated: true)
        }
    }
}
