//
//  InventoryViewController.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/3.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit
final class InventoryViewController: UIViewController {
    @IBOutlet private var tableView: UITableView!
    @IBOutlet private weak var pickerView: UIPickerView!
    @IBOutlet private weak var inventoryBtn: UIButton!
    @IBOutlet private weak var csvOutputBtn: UIBarButtonItem!
    @IBOutlet private weak var speedLabel: UILabel!
    @IBOutlet private weak var tagCountLabel: UILabel!
    @IBOutlet private weak var totalTagCountLabel: UILabel!
    @IBOutlet private weak var displayPcSwitch: UISwitch!
    @IBOutlet private weak var continuousModeSwitch: UISwitch!
    @IBOutlet private weak var reportRssiSwitch: UISwitch!
    @IBOutlet private weak var powerGainPickerTopView: UIView!
    @IBOutlet private weak var powerGainTextField: UITextField!
    @IBOutlet private weak var scrollView: UIScrollView!
    @IBOutlet private weak var scrollViewHeight: NSLayoutConstraint!
    @IBOutlet private weak var operationTimeTextField: UITextField!
    @IBOutlet private weak var optionButton: UIButton!
    @IBOutlet private weak var inventorySessionTextField: UITextField!
    @IBOutlet private weak var sessionFlagTextField: UITextField!
    @IBOutlet private var toDisableElems: [UIControl]!
    private enum PickerViewTag: Int {
        case pickerViewInventorySession = 0
        case pickerViewPowerGain = 1
        case pickerViewSessionFlag = 2
    }
    private weak var appDelegate: AppDelegate?
    private var mTotalTagCount: Int = 0
    private var mTotalTagCounts: Int = 0
    private var currentTagCount: Int = 0
    private var mLastTime: Double = 0
    private var mTagSpeed: Double = 0.0
    private var systemSetting: SystemSetting?
    private var databaseManeger: DatabaseManeger?
    private var isReloadNeeded: Bool = false
    private var isCanScan: Bool = false
    private var isJapan: Bool = false
    private var maskType: MaskType?
    private var sessionFlag: SessionFlag?
    private var inventorySession: SessionType?
    private var aryTags: [AsTagItem] = []
    private var powerGainPickerData: [String] = []
    private var sessionFlagPickerData: [String] = []
    private var inventorySessionPickerData: [String] = []
    private var mapTags: [String: AsTagItem] = [:]
    private var sqlDic: [String: String] = [:]
    override func viewDidLoad() {
        super.viewDidLoad()
        sessionFlagPickerData =  dataSessionFlag
        inventorySessionPickerData =  dataInvenTorySession
        systemSetting = SystemSetting.shared()
        isReloadNeeded = true
        appDelegate = UIApplication.shared.delegate  as? AppDelegate
        appDelegate?.mReader?.setDelegate(self)
        appDelegate?.insertOperation?.maxConcurrentOperationCount = 1
        if appDelegate?.mReader == nil {
            return
        }
        AppDelegate.showGlobalProgressHUD(withTitle: loading)
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        if let inventorySes = appDelegate?.mReader?.inventorySession {
            self.setInventorySession(inventorySes)
        }
        if let sessionFlag = appDelegate?.mReader?.sessionFlag {
            self.setSessionFlag(sessionFlag)
        }
    }
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        NotificationCenter.default.addObserver(self, selector: #selector(asReaderGUNConnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNConnected),
                                               object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(asReaderGUNDisconnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNDisconnected),
                                               object: nil)
        databaseManeger = DatabaseManeger.sharedDatabaseManeger
        databaseManeger?.getDatebaseQueue()?.inDatabase({database in
            if let arr = self.databaseManeger?.getAllAsTagItemsfmdb(database) as? [AsTagItem] {
                self.aryTags = arr
            }
        })
        for item in aryTags {
            mapTags.updateValue(item, forKey: item.tag)
        }
        mTotalTagCount = 0
        mTotalTagCounts = 0
        for item in aryTags {
            mTotalTagCounts += Int(item.count)
        }
        mTagSpeed = 0.0
        isJapan = appDelegate?.mReader?.regionName().hasPrefix("Japan") ?? false
        currentTagCount = 0
        speedLabel.text = "0 tps"
        reportRssiSwitch.isOn = false
        isCanScan = true
        appDelegate?.mReader?.setDelegate(self)
        if !isReloadNeeded {
            return
        }
        isReloadNeeded = true
        maskType = (appDelegate?.mReader?.maskTypeValue).map { MaskType(rawValue: MaskType.RawValue($0)) }
        appDelegate?.mReader?.storedMode = false
        appDelegate?.mReader?.reportMode = false
        initPowerGainPickerData()
        if let powerGain =  appDelegate?.mReader?.powerGain {
            let nPower = powerGain
            self.setPowerGain((Int(nPower / 10)))
        }
        mLastTime = 0
        if let continuousMode = systemSetting?.getContinuousMode() {
            appDelegate?.mReader?.continuousMode = continuousMode
            continuousModeSwitch.isOn = continuousMode
        }
        if let rssiMode = systemSetting?.getRssiMode() {
            appDelegate?.mReader?.rssiMode = rssiMode
            reportRssiSwitch.isOn = rssiMode
        }
        if let disPlayPC = systemSetting?.getDisPlayPC() {
            displayPcSwitch.isOn = disPlayPC
        }
        if let operationTime = appDelegate?.mReader?.operationTime {
            self.setOperationTime("\(operationTime)")
        }
        if let inventory = appDelegate?.mReader?.inventorySession {
            self.setInventorySession(inventory)
        }
        if let sessionFlag = appDelegate?.mReader?.sessionFlag {
            self.setSessionFlag(sessionFlag)
        }
        pickerView.reloadAllComponents()
        updateUI()
        AppDelegate.dismissGlobalHUD()
    }
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name(asreaderGUNDisconnected),
                                                  object: nil)
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name(asreaderGUNConnected),
                                                  object: nil)
        appDelegate?.mReader?.stopSync()
        enableControlByActionState()
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        isReloadNeeded = true
        if segue.identifier == "ReadMemory" {
            let destinationController = segue.destination as? ReadMemoryViewController
            var indexPath = sender as? IndexPath
            if let index = indexPath?.row {
                destinationController?.maskValue = aryTags[index].tag
            }
        } else if segue.identifier == "WriteMemory" {
            let destinationController = segue.destination as? WriteMemoryViewController
            var indexPath = sender as? IndexPath
            if let index = indexPath?.row {
                destinationController?.maskValue = aryTags[index].tag
            }
        } else if segue.identifier == "LockMemory" {
            let destinationController = segue.destination as? TagAccessViewController
            var indexPath = sender as? IndexPath
            if let index = indexPath?.row {
                destinationController?.maskValue = aryTags[index].tag
            }
        }
    }
    @IBAction func prepare(forUnwind segue: UIStoryboardSegue) {
    }
    @IBAction private func powerGainBtnTapped(_ sender: UIButton) {
        pickerView.tag = PickerViewTag.pickerViewPowerGain.rawValue
        pickerView.reloadAllComponents()
        pickerView.isHidden = false
        powerGainPickerTopView.isHidden = false
        guard let text = powerGainTextField.text else {
            return
        }
        guard text.isEmpty else {
            if let row = powerGainPickerData.index(of: text) {
                pickerView.selectRow(row, inComponent: 0, animated: false)
                return
            }
            pickerView.selectRow(0, inComponent: 0, animated: false)
            return
        }
    }
    @IBAction private func displayPcSwitchTapped(_ sender: UISwitch) {
        systemSetting?.setDisPlayPC(displayPcSwitch.isOn)
        tableView.reloadData()
    }
    @IBAction private func continuousModeSwitchTapped(_ sender: UISwitch) {
        appDelegate?.mReader?.continuousMode = continuousModeSwitch.isOn
        systemSetting?.setContinuousMode(continuousModeSwitch.isOn)
    }
    @IBAction private func reportRssiSwitchTapped(_ sender: UISwitch) {
        appDelegate?.mReader?.rssiMode = reportRssiSwitch.isOn
        systemSetting?.setRssiMode(reportRssiSwitch.isOn)
        aryTags.removeAll()
        mapTags.removeAll()
        appDelegate?.insertOperation?.cancelAllOperations()
        databaseManeger?.getDatebaseQueue()?.inTransaction({(dataBase, rollback) in
            if let result = self.databaseManeger?.deleteAll(asTagItemsFmdb: dataBase) {
                if !result {
                    let isRollback = rollback
                    isRollback?.pointee = true
                }
            }
        })
        currentTagCount = 0
        self.tagCountLabel.text = "\(aryTags.count)"
        self.totalTagCountLabel.text = "\(aryTags.count)"
        self.speedLabel.text = "0 tps"
        mTagSpeed = 0.0
        mLastTime = 0
        mTotalTagCount = 0
        mTotalTagCounts = 0
        self.tableView.reloadData()
    }
    @IBAction private func inventoryBtnTapped(_ sender: UIButton) {
        if appDelegate?.mReader?.getAction() == CommandStop {
            appDelegate?.mReader?.inventory()
        } else {
            appDelegate?.mReader?.stop()
        }
    }
    @IBAction private func clearBtnTapped(_ sender: UIButton) {
        aryTags.removeAll()
        mapTags.removeAll()
        tagCountLabel.text = "\(aryTags.count)"
        totalTagCountLabel.text = "\(aryTags.count)"
        speedLabel.text = "0 tps"
        mTagSpeed = 0.0
        mLastTime = 0
        mTotalTagCount = 0
        currentTagCount = 0
        mTotalTagCounts = 0
        appDelegate?.insertOperation?.cancelAllOperations()
        databaseManeger?.getDatebaseQueue()?.inTransaction({dataBase, rollback in
            if let result = self.databaseManeger?.deleteAll(asTagItemsFmdb: dataBase) {
                if !result {
                    let isRollback = rollback
                    isRollback?.pointee = true
                }
            }
        })
        tableView.reloadData()
    }
    @IBAction private func maskBtnTapped(_ sender: UIButton) {
        if maskType == MaskType_NO_MASK {
            let alertController = UIAlertController(title: "",
                                                    message: "Please set the mask type in RFID Options menu before using mask.",
                                                    preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
            return
        } else if maskType == MaskType_Selection {
            nowViewWillDisappear()
            performSegue(withIdentifier: "SelectionMask", sender: self)
        } else if maskType == MaskType_EPC {
            nowViewWillDisappear()
            performSegue(withIdentifier: "EpcMask", sender: self)
        }
        isReloadNeeded = false
    }
    @IBAction private func pickerCloseBtnTapped(_ sender: UIButton) {
        hidePickerView()
    }
    @IBAction private func powerGainPickerOkBtnTapped(_ sender: UIButton) {
        if let pickerViewTag = PickerViewTag(rawValue: pickerView.tag) {
            if pickerViewTag == .pickerViewPowerGain {
                var idx: Int32 = Int32(pickerView.selectedRow(inComponent: 0))
                if let min = appDelegate?.mReader?.powerGainRange().min {
                    idx += (min / 10)
                }
                self.setPowerGain(Int(idx))
                appDelegate?.mReader?.powerGain = idx * 10
                systemSetting?.setPowerGain(Int(idx * 10))
            } else if pickerViewTag == .pickerViewInventorySession {
                self.setInventorySession(SessionType(rawValue: SessionType.RawValue(pickerView.selectedRow(inComponent: 0))))
                if let inventory = inventorySession {
                    appDelegate?.mReader?.inventorySession = inventory
                    systemSetting?.setInventorySession(Int(inventory.rawValue))
                }
            } else if pickerViewTag == .pickerViewSessionFlag {
                self.setSessionFlag(SessionFlag(rawValue: SessionFlag.RawValue(pickerView.selectedRow(inComponent: 0))))
                if let aSessionFlag = sessionFlag {
                    appDelegate?.mReader?.sessionFlag = aSessionFlag
                    systemSetting?.setSessionFlag(Int(aSessionFlag.rawValue))
                }
            }
        }
        hidePickerView()
    }
    @IBAction private func operationTimeBtnTapped(_ sender: UIButton) {
        let alertController = UIAlertController(title: "Operation Time:", message: "Please enter operation time.\n(ex: 30 ms)", preferredStyle: .alert)
        alertController.addTextField { (textField) in
            textField.keyboardType = .numberPad
            textField.delegate = self
        }
        let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            var value = alertController.textFields?.first?.text
            guard let text = value else {
                return
            }
            if text.isEmpty {
                value = "0"
            }
            if let aValue = value {
                self.setOperationTime(aValue)
                self.appDelegate?.mReader?.operationTime = Int32(aValue) ?? 0
                self.systemSetting?.setOperationTime(Int(aValue) ?? 0)
            }
        })
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    @IBAction private func csvOutputBtnTapped(_ sender: UIBarButtonItem) {
        DispatchQueue.main.async {
            var rfidCount: Int = 0
            for index in 0..<self.aryTags.count {
                let item: AsTagItem? = self.aryTags[index]
                if item?.tag != nil {
                    rfidCount += 1
                }
            }
            let fileName = "rfid.csv"
            if rfidCount != 0 {
                if CSVData.isFileExist(fileName) {
                    CSVData.deleteFile(forFileName: fileName)
                }
                CSVData.createCSV(withFileName: fileName)
                CSVData.exportRfidCSV(fileName, withArray: self.aryTags, isEpc: self.displayPcSwitch.isOn)
            } else {
                return
            }
        }
    }
    @IBAction private func backBtnTapped(_ sender: UIButton) {
        nowViewWillDisappear()
        navigationController?.popViewController(animated: true)
    }
    @IBAction private func onClickOptionButton(_ sender: UIButton) {
        if self.scrollViewHeight.constant > 0 {
            self.scrollViewHeight.constant = 0
        } else {
            self.scrollViewHeight.constant = 275
        }
    }
    @IBAction private func onClickInventorySessionButton(_ sender: UIButton) {
        pickerView.tag = PickerViewTag.pickerViewInventorySession.rawValue
        pickerView.reloadAllComponents()
        if let inventoryRaw =  inventorySession?.rawValue {
            pickerView.selectRow(Int(inventoryRaw), inComponent: 0, animated: false)
        }
        pickerView.isHidden = false
        powerGainPickerTopView.isHidden = false
    }
    @IBAction private func onClickSessionFlagButton(_ sender: UIButton) {
        pickerView.tag = PickerViewTag.pickerViewSessionFlag.rawValue
        pickerView.reloadAllComponents()
        if let sessionValue = sessionFlag?.rawValue {
            pickerView.selectRow(Int(sessionValue), inComponent: 0, animated: false)
        }
        pickerView.isHidden = false
        powerGainPickerTopView.isHidden = false
    }
}
// MARK: - UITableViewDelegate, UITableViewDataSource
extension InventoryViewController: UITableViewDelegate, UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return aryTags.count
    }
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "InventoryCell", for: indexPath) as? InventoryTableViewCell
        var item: AsTagItem?
        item = aryTags[indexPath.row]
        if item != nil {
            if displayPcSwitch.isOn {
                guard let tag = item?.tag else {
                    return UITableViewCell()
                }
                cell?.setTagValue(tag)
            } else {
                if let tagStr = item?.tag {
                    let tagLabelStr = String(tagStr.suffix(from: tagStr.index(tagStr.startIndex, offsetBy: Int(TAG_PC_LENGTH))))
                    cell?.setTagValue(tagLabelStr)
                }
            }
            if let aCount = item?.count {
                cell?.setCountValue("\(aCount)")
            }
            if reportRssiSwitch.isOn {
                if let arssi = item?.rssi, let aPhase = item?.phase, let aFrequency = item?.frequency {
                    cell?.setRssiValue(String(format: "%.1f dB", arssi))
                    cell?.setPhaseValue(String(format: "%.0f˚", aPhase))
                    cell?.setFrequencyValue(String(format: "%.2fMHz ", aFrequency))
                 } else {
                    cell?.setRssiValue("")
                    cell?.setPhaseValue("")
                    cell?.setFrequencyValue("")
                }
            } else {
                cell?.setRssiValue("")
                cell?.setPhaseValue("")
                cell?.setFrequencyValue("")
            }
        }
        
        guard let aCell = cell else {
            return UITableViewCell()
        }
        return aCell
    }
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if appDelegate?.mReader?.getAction() != CommandStop {
            return
        }
        let alert = UIAlertController(title: "Additional Actions",
                                      message: "You can choose one of the following actions for this Tag",
                                      preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "Read Memory", style: .default, handler: { _ in
            self.nowViewWillDisappear()
            self.performSegue(withIdentifier: "ReadMemory", sender: indexPath)
        }))
        alert.addAction(UIAlertAction(title: "Write Memory", style: .default, handler: { _ in
            self.nowViewWillDisappear()
            self.performSegue(withIdentifier: "WriteMemory", sender: indexPath)
        }))
        alert.addAction(UIAlertAction(title: "Lock Memory", style: .default, handler: { _ in
            self.nowViewWillDisappear()
            self.performSegue(withIdentifier: "LockMemory", sender: indexPath)
        }))
        alert.addAction(UIAlertAction(title: cancel, style: .cancel, handler: { _ in
        }))
        present(alert, animated: true)
    }
}
// MARK: - UIPickerViewDelegate, UIPickerViewDataSource
extension InventoryViewController: UIPickerViewDataSource, UIPickerViewDelegate {
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if let pickerViewTag = PickerViewTag(rawValue: pickerView.tag) {
            if pickerViewTag == .pickerViewPowerGain {
                return powerGainPickerData.count
            } else if pickerViewTag == .pickerViewInventorySession {
                return inventorySessionPickerData.count
            } else if pickerViewTag == .pickerViewSessionFlag {
                return sessionFlagPickerData.count
            }
        }
        return 0
    }
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if let pickerViewTag = PickerViewTag(rawValue: pickerView.tag) {
            if pickerViewTag == .pickerViewPowerGain {
                return powerGainPickerData[row]
            } else if pickerViewTag == .pickerViewInventorySession {
                return inventorySessionPickerData[row]
            } else if pickerViewTag == .pickerViewSessionFlag {
                return sessionFlagPickerData[row]
            }
        }

        return "None"
    }
}
// MARK: - UITextFieldDelegate
extension InventoryViewController: UITextFieldDelegate {
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        let text = textField.text ?? ""
        let length = text.count + string.count - range.length
        return length <= 9
    }
}
// MARK: - Custom
extension InventoryViewController {
    private func updateUI() {
        tagCountLabel.text = "\(aryTags.count)"
        totalTagCountLabel.text = "\(mTotalTagCounts)"
        speedLabel.text = String(format: "%0.2f tps", mTagSpeed)
        tableView.reloadData()
    }
    private func initPowerGainPickerData() {
        var tmpArray: [String] = []
        if let min = appDelegate?.mReader?.powerGainRange().min,
            let max = appDelegate?.mReader?.powerGainRange().max {
            var index: Int = Int(min)
            while index <= max {
                tmpArray.append(String(format: "%i.0 dB", index / 10))
                index += 10
            }
        }
        powerGainPickerData = tmpArray
    }
    private func hidePickerView() {
        powerGainPickerTopView.isHidden = true
        pickerView.isHidden = true
    }
    @objc private func changStatus() {
        isCanScan = true
    }
    private func enableControlByActionState() {
        if appDelegate?.mReader?.getAction() == CommandStop {
            replaceDB()
            inventoryBtn.setTitle("Inventory", for: .normal)
            mLastTime = 0
            currentTagCount = mTotalTagCount
            for value in toDisableElems {
                value.isEnabled = true
            }
            csvOutputBtn.isEnabled = true
            tableView.isUserInteractionEnabled = true
        }
    }
    private func nowViewWillDisappear() {
        replaceDB()
        if let operationCount = appDelegate?.insertOperation?.operationCount {
            if operationCount > 0 {
                AppDelegate.dismissGlobalHUD()
                AppDelegate.showGlobalProgressHUD(withTitle: "Data Insert...")
                CATransaction.commit()
                print("wait")
                while true {
                    Thread.sleep(forTimeInterval: 0.5)
                    if appDelegate?.insertOperation?.operationCount == 0 {
                        print("wait")
                        break
                    }
                }
                AppDelegate.dismissGlobalHUD()
                CATransaction.commit()
             } else {
                return
            }
         } else {
            return
        }
    }
    private func replaceDB() {
        if sqlDic.keys.isEmpty {
            return
        }
        let dic = sqlDic
        sqlDic.removeAll()
        appDelegate?.insertOperation?.addOperation({
            self.databaseManeger?.getDatebaseQueue()?.inTransaction({ database, rollback in
                for (_, value) in dic {
                    if let result = self.databaseManeger?.updataOrAddSql(value, fmdb: database) {
                        if !result {
                            let isRollback = rollback
                            isRollback?.pointee = true
                        }
                    }
                }
            })
        })
    }
}
// MARK: - AsReaderDelegate
extension InventoryViewController: AsReaderDelegate {
    func readerInitialized(_ reader: AsReader?) {
        if appDelegate?.mReader?.isBarcodeModule != nil {
            AppDelegate.showGlobalProgressHUD(withTitle: "Barcode Setting...")
            CATransaction.commit()
            SystemSetting.shared()?.barcodeSetting()
            AppDelegate.dismissGlobalHUD()
            CATransaction.commit()
        }
        if appDelegate?.mReader?.isRFIDModule != nil {
            AppDelegate.showGlobalProgressHUD(withTitle: "RFID Setting...")
            CATransaction.commit()
            SystemSetting.shared()?.rfidSetting()
            AppDelegate.dismissGlobalHUD()
            CATransaction.commit()
        }
    }
    func on(asReaderTriggerKeyEvent status: Bool) -> Bool {
        if appDelegate?.mReader?.getScanMode() == RFIDScanMode {
            if status {
                if isCanScan {
                    appDelegate?.mReader?.inventory()
                }
            } else {
                appDelegate?.mReader?.stop()
            }
            return false
        } else {
            return true
        }
    }
    func on(asReaderLeftModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func on(asReaderRightModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func updateDeviceState(_ error: ResultCode) {
    }
    func readTag(_ tag: String?, rssi: Float, phase: Float, frequency: Float) {
        var rssiBlock: Float = rssi
        var phaseBlock: Float = phase
        let frequencyBlock: Float = frequency
        let queue = DispatchQueue(label: "queue")
        queue.async {
            let time = CFAbsoluteTimeGetCurrent()
            var interval: Double = 0.0
            self.mTotalTagCount += 1
            self.mTotalTagCounts += 1
            if self.mLastTime == 0 {
                self.mTagSpeed = 0.0
                self.mLastTime = time
            } else {
                interval = Double(time - self.mLastTime)
                if interval != 0 {
                    self.mTagSpeed = Double(self.mTotalTagCount - self.currentTagCount) / interval
                }
            }
            if let getRssiMode = self.systemSetting?.getRssiMode() {
                if !getRssiMode {
                    rssiBlock = 0.0
                    phaseBlock = 0.0
                }
            }
            var item: AsTagItem?
            let lockQueue = DispatchQueue(label: "self")
            lockQueue.sync {
                if let aTag = tag {
                    item = self.mapTags[aTag]
                }
                if let aItem = item {
                    item?.count = aItem.count + 1
                    item?.updateRssi(rssiBlock, withPhase: phaseBlock, frequency: frequencyBlock)
                 } else {
                    item = AsTagItem.item(with: tag, rssi: rssiBlock, phase: phaseBlock, frequency: frequencyBlock) as? AsTagItem
                    if let aItem = item {
                        self.aryTags.append(aItem)
                        self.mapTags.updateValue(aItem, forKey: tag ?? "")
                    }
                }
                 if let aItem = item {
                    let sql: String = "REPLACE INTO AsTagItem(tag,count,rssi,phase,frequency,datetime)VALUES ('\(aItem.tag)','\(aItem.count)','\(aItem.rssi)','\(aItem.phase)','\(aItem.frequency)','\(aItem.dateTime)')"
                    self.sqlDic.updateValue(sql, forKey: aItem.tag)
                    if self.sqlDic.values.count > 20 {
                        self.replaceDB()
                    }
                }
                DispatchQueue.main.async {
                    self.tagCountLabel.text = "\(self.aryTags.count)"
                    self.totalTagCountLabel.text = "\(self.mTotalTagCounts)"
                    self.speedLabel.text = String(format: "%0.2f tps", self.mTagSpeed)
                    self.tableView.reloadData()
                }
            }
        }
    }
    func changedActionState(_ action: CommandType, resultCode: Int) {
        if action == CommandStop {
            replaceDB()
            inventoryBtn.setTitle("Inventory", for: .normal)
            mLastTime = 0
            currentTagCount = mTotalTagCount
            for value in toDisableElems {
                value.isEnabled = true
            }
            csvOutputBtn.isEnabled = true
            tableView.isUserInteractionEnabled = true
            if resultCode == Int(ResultNoError.rawValue) {
                isCanScan = true
                NSObject.cancelPreviousPerformRequests(withTarget: self, selector: #selector(changStatus), object: nil)
            } else {
                isCanScan = false
                NSObject.cancelPreviousPerformRequests(withTarget: self, selector: #selector(changStatus), object: nil)
                perform(#selector(changStatus), with: nil, afterDelay: 1.0)
            }
         } else {
            if resultCode == Int(ResultNoError.rawValue) {
                inventoryBtn.setTitle("Stop", for: .normal)
                for value in toDisableElems {
                    value.isEnabled = false
                }
                csvOutputBtn.isEnabled = false
                tableView.isUserInteractionEnabled = false
            }
        }
    }
}
// MARK: - NSNotification
extension InventoryViewController {
    @objc private func asReaderGUNDisconnected(_ notification: Notification?) {
        DispatchQueue.main.async {
            self.appDelegate?.mReader = nil
            self.nowViewWillDisappear()
            self.navigationController?.popToRootViewController(animated: true)
        }
    }
    @objc private func asReaderGUNConnected(_ notification: Notification?) {
        DispatchQueue.main.async {
            AppDelegate.showGlobalProgressHUD(withTitle: loading)
            CATransaction.commit()
            self.appDelegate?.mReader? = AsReader(asReaderGUN: self.appDelegate?.mAsReaderGUN, delegate: self)
            AppDelegate.dismissGlobalHUD()
            CATransaction.commit()
        }
    }
}
// MARK: - Setters
extension InventoryViewController {
    private func setInventorySession(_ inventorySession: SessionType) {
        let index: Int = Int(inventorySession.rawValue)
        inventorySessionTextField.text = inventorySessionPickerData[index]
        self.inventorySession = inventorySession
    }
    private func setPowerGain(_ power: Int) {
        let value = String(format: "%li.0 dB", power)
        if powerGainPickerData.contains(value) {
            if let index = powerGainPickerData.index(of: value) {
                if powerGainPickerData.count > index {
                    powerGainTextField.text = powerGainPickerData[index]
                }
            }
        }
    }
    private func setSessionFlag(_ sessionFlag: SessionFlag) {
        let index: Int = Int(sessionFlag.rawValue)
        sessionFlagTextField.text = sessionFlagPickerData[index]
        self.sessionFlag = sessionFlag
    }
    private func setOperationTime(_ operationTime: String?) {
        operationTimeTextField.text = operationTime
    }
}
