//
//  MainViewController.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/2.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit
import AsReaderGunSDK
import CoreBluetooth
import AsRingAccessorySDK

final class MainViewController: UIViewController {
    @IBOutlet private weak var asRingPlusVersion: UILabel!
    @IBOutlet private var buttonCollection: [MainButton]!
    @IBOutlet private weak var batteryImage: UIImageView!
    @IBOutlet private weak var vibratorTextField: UITextField!
    @IBOutlet private weak var buzzerTextField: UITextField!
    @IBOutlet private weak var buzzerButton: UIButton!
    @IBOutlet private weak var vibratorButton: UIButton!
    @IBOutlet private weak var logoImage: UIImageView!
    @IBOutlet private weak var labelDemoVer: UILabel!
    @IBOutlet private weak var labelFirmwareVer: UILabel!
    @IBOutlet private weak var labelAddress: UILabel!
    @IBOutlet private weak var regionLabel: UILabel!
    @IBOutlet private weak var snLabel: UILabel!
    @IBOutlet private weak var ufVersion: UILabel!
    @IBOutlet private weak var mainScrollView: UIScrollView!
    private weak var readerDelegate: AsReaderDelegate?
    private weak var appDelegate: AppDelegate?
    private var searchBarButtonItem: UIBarButtonItem?
    private var logoDebugTap: UITapGestureRecognizer?
    private var asRingDeviceSDK: AsRingDeviceSDK?
    private var backView: UIView?
    private var updateUrl = ""
    private enum BatteryStatus: Int {
        case batteryNone, batteryQuarter, batteryHalf, batteryThirdQuarter, batteryFull
    }
    private enum ConnectionStatus: Int {
        case connectionOff, connectionOn
    }
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate  as? AppDelegate
        appDelegate?.mAsReaderGUN = AsReaderGUN(deviceModel: deviceModel)
        appDelegate?.mAsReaderGUN?.deviceModel = deviceModel
        labelDemoVer.text = appDelegate?.getAppVersion()
        regionLabel.text = ""
        snLabel.text = ""
        ufVersion.text = ""
        mainScrollView.contentSize = CGSize(width: 0, height: view.frame.size.height)
        mainScrollView.showsVerticalScrollIndicator = false
        searchBarButtonItem = navigationItem.rightBarButtonItem
        NotificationCenter.default.addObserver(self, selector: #selector(asReaderGUNDisconnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNDisconnected),
                                               object: nil)
        logoDebugTap = nil
        DispatchQueue.global(qos: .default).async {
            VersionHandler.shared()?.versionHandlerDelegate = self
            VersionHandler.shared()?.getServeVersion(withUrl:
                "https://www.asteriskweb.jp/mrxapp/AsReaderLR/manifest.plist")
        }
        if let buzzerTime = SystemSetting.shared()?.getBuzzerTime() {
            buzzerTextField.text = "\(buzzerTime)"
        }
        if let vibratorTime = SystemSetting.shared()?.getVibratorTime() {
            vibratorTextField.text = "\(vibratorTime)"
        }
        buzzerTextField.delegate = self
        vibratorTextField.delegate = self
        backView = UIView()
        backView?.backgroundColor = UIColor.gray
        backView?.alpha = 0.5
        backView?.frame = CGRect(x: 0, y: 0, width: view.frame.size.width, height: view.frame.size.height)
        mainScrollView.setContentOffset(CGPoint(x: 0, y: 0), animated: true)
        let tapGestureRecognizer = UITapGestureRecognizer.init(target: self, action: #selector(keyboardHide))
        tapGestureRecognizer.cancelsTouchesInView = false
        backView?.addGestureRecognizer(tapGestureRecognizer)
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        NotificationCenter.default.addObserver(self, selector: #selector(asReaderGUNConnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNConnected),
                                                       object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(asReaderGUNDisconnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNDisconnected),
                                                       object: nil)
    }
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        connectMethod()
        if  view.frame.size.height < 590 {
            mainScrollView.contentSize = CGSize(width: 0, height: 590)
        }
    }
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name(asreaderGUNConnected),
                                                      object: nil)
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name(asreaderGUNDisconnected),
                                                      object: nil)
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    // MARK: - IBActions
    @IBAction private func onClickVibratorButton(_ sender: UIButton) {
        guard let vibratorStr =  vibratorTextField.text,
              let vibratorIndex = Int(vibratorStr) else {
                return
        }
        if vibratorIndex > 0 && vibratorIndex < 5000 {
            SystemSetting.shared()?.setVibratorTime(vibratorIndex)
            if let resultCode = appDelegate?.mReader?.startVibrator(withVibratorTime: Int32(vibratorIndex)) {
                if resultCode == ResultOperationFailed {
                    print("Vibrator Start ERROR!")
                }
            }
        } else {
            let alertController = UIAlertController(title: "Vibrator Value", message: "Please enter the value (1~4999)", preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .cancel, handler: nil)
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
        }
    }
    @IBAction private func onClickBuzzerButton(_ sender: UIButton) {
        guard let buzzerStr: String =  buzzerTextField.text,
              let buzzerIndex: Int = Int(buzzerStr) else {
                return
        }
        if buzzerIndex > 0 && buzzerIndex < 5000 {
            SystemSetting.shared()?.setBuzzerTime(buzzerIndex)
            if let resultCode = appDelegate?.mReader?.startBuzzer(withBuzzerTime: Int32(buzzerIndex)) {
                if resultCode == ResultOperationFailed {
                    print("Buzzer Start ERROR!")
                }
            }
        } else {
            let alertController = UIAlertController(title: "Buzzer Value", message: "Please enter the value (1~4999)", preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .cancel, handler: nil)
            alertController.addAction(okAction)
            present(alertController, animated: true, completion: nil)
        }
    }
    @IBAction private func prepare(forUnwind segue: UIStoryboardSegue) {
    }
}
// MARK: - Custom
extension MainViewController {
    private func disconnectDevice() {
        if appDelegate?.mReader != nil {
            appDelegate?.mReader?.disconnect()
        }
        appDelegate?.mReader = nil
        disableAllUIs()
    }
    @objc private func keyboardHide() {
        mainScrollView.isScrollEnabled = true
        backView?.removeFromSuperview()
        vibratorTextField.resignFirstResponder()
        buzzerTextField.resignFirstResponder()
        if let  buzzerStr: String =  buzzerTextField.text {
            if buzzerStr.isEmpty {
                buzzerTextField.text = "1"
            } else {
                if let buzzerIndex: Int = Int(buzzerStr) {
                    if buzzerIndex == 0 {
                        buzzerTextField.text = "1"
                    }
                    if buzzerIndex >= 5000 {
                        buzzerTextField.text = "4999"
                        let alertController = UIAlertController(title: "Buzzer Value",
                                                                message: "Please enter the value (1~4999)",
                                                                preferredStyle: .alert)
                        let actionCancel = UIAlertAction(title: ok, style: .cancel, handler: nil)
                        alertController.addAction(actionCancel)
                        navigationController?.present(alertController, animated: false)
                    }
                    SystemSetting.shared()?.setBuzzerTime(buzzerIndex)
                }
            }
        }
        if let  vibratorStr: String =  vibratorTextField.text {
            if vibratorStr.isEmpty {
                vibratorTextField.text = "1"
            } else {
                if let vibratorIndex: Int = Int(vibratorStr) {
                    if vibratorIndex == 0 {
                        vibratorTextField.text = "1"
                    }
                    if vibratorIndex >= 5000 {
                        vibratorTextField.text = "4999"
                        let alertController = UIAlertController(title: "Vibrator Value",
                                                                message: "Please enter the value (1~4999)",
                                                                preferredStyle: .alert)
                        let actionCancel = UIAlertAction(title: ok, style: .cancel, handler: nil)
                        alertController.addAction(actionCancel)
                        navigationController?.present(alertController, animated: false)
                    }
                    SystemSetting.shared()?.setVibratorTime(vibratorIndex)
                }
            }
        }
        
        if mainScrollView.contentSize.height - mainScrollView.bounds.size.height > 0 {
            mainScrollView.setContentOffset(CGPoint(x: 0, y: mainScrollView.contentSize.height - mainScrollView.bounds.size.height), animated: false)
        } else {
            mainScrollView.setContentOffset(CGPoint(x: 0, y: 0), animated: true)
        }
    }
    private func enableControlByActionState() {
    }
    private func enableAllButtons() {
        for btn in buttonCollection {
            btn.enableButton()
        }
        changeLogoImage(ConnectionStatus.connectionOn.rawValue)
        buzzerButton.setTitleColor(UIColor.black, for: .normal)
        vibratorButton.setTitleColor(UIColor.black, for: .normal)
        if let buzzerTime = SystemSetting.shared()?.getBuzzerTime() {
            buzzerTextField.text = "\(buzzerTime)"
        }
        if let vibratorTime = SystemSetting.shared()?.getVibratorTime() {
            vibratorTextField.text = "\(vibratorTime)"
        }
        buzzerTextField.isEnabled = true
        vibratorTextField.isEnabled = true
        buzzerButton.isEnabled = true
        vibratorButton.isEnabled = true
    }
    private func disableAllUIs() {
        keyboardHide()
        mainScrollView.setContentOffset(CGPoint(x: 0, y: 0), animated: false)
        asRingPlusVersion.text = ""
        labelFirmwareVer.text = ""
        labelAddress.text = ""
        regionLabel.text = ""
        snLabel.text = ""
        ufVersion.text = ""
        buzzerButton.setTitleColor(UIColor.gray, for: .normal)
        vibratorButton.setTitleColor(UIColor.gray, for: .normal)
        if let buzzerTime = SystemSetting.shared()?.getBuzzerTime() {
            buzzerTextField.text = "\(buzzerTime)"
        }
        if let vibratorTime = SystemSetting.shared()?.getVibratorTime() {
            vibratorTextField.text = "\(vibratorTime)"
        }
        buzzerTextField.isEnabled = false
        vibratorTextField.isEnabled = false
        buzzerButton.isEnabled = false
        vibratorButton.isEnabled = false
        for btn in buttonCollection {
            btn.disableButton()
        }
        changeBatteryImage(BatteryStatus.batteryNone.rawValue)
        changeLogoImage(ConnectionStatus.connectionOff.rawValue)
        rightBarButtonDisconnected()
    }
    private func disableBarcodeButtons() {
        buttonCollection[6].disableButton()
        buttonCollection[7].disableButton()
    }
    private func disableRFIDButtons() {
        buttonCollection[0].disableButton()
        buttonCollection[1].disableButton()
        buttonCollection[2].disableButton()
        buttonCollection[3].disableButton()
        buttonCollection[4].disableButton()
        buttonCollection[5].disableButton()
        buttonCollection[8].disableButton()
    }
    private func changeBatteryImage(_ batteryStatus: Int) {
        if let status = BatteryStatus(rawValue: batteryStatus) {
            if status == .batteryNone {
                batteryImage.image = UIImage(named: "battery0")
            } else if status == .batteryQuarter {
                batteryImage.image = UIImage(named: "battery1")
            } else if status == .batteryHalf {
                batteryImage.image = UIImage(named: "battery2")
            } else if status == .batteryThirdQuarter {
                batteryImage.image = UIImage(named: "battery3")
            } else if status == .batteryFull {
                batteryImage.image = UIImage(named: "battery4")
            }
        }
    }
    private func changeLogoImage(_ connectionStatus: Int) {
        if let status = ConnectionStatus(rawValue: connectionStatus) {
            if status == .connectionOff {
                logoImage.image = UIImage(named: "asreader_logo_off")
                if let aLogoDebugTap = logoDebugTap {
                    logoImage.isUserInteractionEnabled = false
                    logoImage.removeGestureRecognizer(aLogoDebugTap)
                    logoDebugTap = nil
                }
            } else if status == .connectionOn {
                logoImage.image = UIImage(named: "asreader_logo_on")
                if logoDebugTap != nil {
                    logoDebugTap = nil
                }
                logoDebugTap = UITapGestureRecognizer(target: self, action: #selector(logoDebugTapDetected(_:)))
                logoDebugTap?.numberOfTapsRequired = 5
                logoImage.isUserInteractionEnabled = true
                if let aLogoDebugTap = logoDebugTap {
                    logoImage.addGestureRecognizer(aLogoDebugTap)
                }
            }
        }
    }
    private func rightBarButtonDisconnected() {
        navigationItem.rightBarButtonItem = searchBarButtonItem
    }
    @objc private func logoDebugTapDetected(_ sender: UITapGestureRecognizer?) {
        if sender?.state == .ended {
            let actionSheet = UIActionSheet(title: "Please select the operationg mode.",
                                            delegate: self,
                                            cancelButtonTitle: "Debug Mode",
                                            destructiveButtonTitle: "Normal Mode")
            actionSheet.tag = 200
            actionSheet.show(in: view)
        }
    }
    private func connectMethod() {
        if appDelegate?.mReader != nil {
            asRingPlusVersion.text = AsRingDeviceSDK.shared().getAsRingFirmwareRevision()
            print("Loading shows")
            AppDelegate.showGlobalProgressHUD(withTitle: loading)
            appDelegate?.mReader?.setDelegate(self)
            labelFirmwareVer.text = appDelegate?.mReader?.firmwareVersion()
            labelAddress.text = appDelegate?.mReader?.mAsReaderGUN.address()
            if let batteryStatus = appDelegate?.mReader?.batteryStatus() {
                changeBatteryImage(Int(batteryStatus))
            }
            enableAllButtons()
            if let isBarcodeModule = appDelegate?.mReader?.isBarcodeModule() {
                if !isBarcodeModule {
                    disableBarcodeButtons()
                }
            }
            if let isRFIDModule = appDelegate?.mReader?.isRFIDModule() {
                if !isRFIDModule {
                    disableRFIDButtons()
                }
            }
            regionLabel.text = appDelegate?.mReader?.regionName()
            snLabel.text = appDelegate?.mReader?.serialNumber
            snLabel.text = appDelegate?.mReader?.serialNumber
            ufVersion.text = appDelegate?.mReader?.rFModuleVersion()
            print("Loading hide")
            SystemSetting.shared()?.asReaderGUNSetting()
            AppDelegate.dismissGlobalHUD()
        } else {
            disableAllUIs()
        }
    }
}
// MARK: - UITextFieldDelegate
extension MainViewController: UITextFieldDelegate {
    func textFieldDidBeginEditing(_ textField: UITextField) {
        mainScrollView.isScrollEnabled = false
        if let aBackView = backView {
            view.addSubview(aBackView)
        }
        mainScrollView.setContentOffset(CGPoint(x: 0, y: 300), animated: true)
    }
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange,
                   replacementString string: String) -> Bool {
        if string == "\n" {
            keyboardHide()
         } else {
            if let text = textField.text {
                if Int(text+string) ?? 0 <= 0 {
                    textField.text = "1"
                }
            }
        }
        return true
    }
}
// MARK: - UIActionSheetDelegate
extension MainViewController: UIActionSheetDelegate {
    func actionSheet(_ actionSheet: UIActionSheet, clickedButtonAt buttonIndex: Int) {
        if actionSheet.tag == 200 {
            if buttonIndex == 0 {
                appDelegate?.mReader?.setDebugMode(false)
            } else {
                appDelegate?.mReader?.setDebugMode(true)
            }
        }
    }
}
// MARK: - VersionHandlerDelegate
extension MainViewController: VersionHandlerDelegate {
    func updateUrl(_ url: String?, forceType: String?) {
        DispatchQueue.main.async {
            if let updateUrl = url {
                self.updateUrl = updateUrl
            }
            if forceType == "0" {
                let message = "There is a new version of the Application server, please go to update the latest Application server"
                let alertController = UIAlertController(title: "Warning", message: message, preferredStyle: .alert)
                let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
                let otherAction = UIAlertAction(title: "Update", style: .default, handler: { _ in
                    if let anUrl = URL(string: self.updateUrl) {
                        if #available(iOS 10.0, *) {
                            UIApplication.shared.open(anUrl, options: [:], completionHandler: { _ in
                            })
                        } else {
                            if UIApplication.shared.canOpenURL(anUrl) {
                                UIApplication.shared.openURL(anUrl)
                            }
                        }
                    }
                })
                alertController.addAction(cancelAction)
                alertController.addAction(otherAction)
                self.present(alertController, animated: true, completion: nil)
            } else {
                 let message = "Your Application version is too old, it is necessary to update the latest version of the Application server"
                let alertController = UIAlertController(title: "Warning", message: message, preferredStyle: .alert)
                let cancelAction = UIAlertAction(title: "Update", style: .cancel, handler: { _ in
                    if let anUrl = URL(string: self.updateUrl) {
                        if #available(iOS 10.0, *) {
                            UIApplication.shared.open(anUrl, options: [:], completionHandler: { _ in
                            })
                        } else {
                            if UIApplication.shared.canOpenURL(anUrl) {
                                UIApplication.shared.openURL(anUrl)
                            }
                        }
                    }
                })
                alertController.addAction(cancelAction)
                self.present(alertController, animated: true, completion: nil)
            }
        }
    }
}
// MARK: - AsReaderDelegate
extension MainViewController: AsReaderDelegate {
    func on(asReaderTriggerKeyEvent status: Bool) -> Bool {
        return false
    }
    func on(asReaderLeftModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func on(asReaderRightModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func readerInitialized(_ reader: AsReader?) {
        enableAllButtons()
        if let isBarcodeModule = appDelegate?.mReader?.isBarcodeModule() {
            if !isBarcodeModule {
                disableBarcodeButtons()
            } else {
                AppDelegate.showGlobalProgressHUD(withTitle: "Barcode Setting...")
                CATransaction.commit()
                SystemSetting.shared()?.barcodeSetting()
                AppDelegate.dismissGlobalHUD()
                CATransaction.commit()
            }
        }
        if let isRFIDModule = appDelegate?.mReader?.isRFIDModule() {
            if !isRFIDModule {
                disableRFIDButtons()
            } else {
                AppDelegate.showGlobalProgressHUD(withTitle: "RFID Setting...")
                CATransaction.commit()
                SystemSetting.shared()?.rfidSetting()
                AppDelegate.dismissGlobalHUD()
                CATransaction.commit()
            }
        }
    }
    func updateDeviceState(_ error: ResultCode) {
    }
    func readTag(_ tag: String?, rssi: Float, phase: Float) {
    }
    func changedActionState(_ action: CommandType, resultCode: Int) {
        enableControlByActionState()
    }
    func accessResult(_ error: ResultCode, actionState action: CommandType, epc: String?, data: String?, rssi: Float, phase: Float, frequency: Float) {
        if error != ResultNoError {
            let alertController = UIAlertController(title: "Error", message: AsResultCode.msg(error), preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: ok, style: .cancel, handler: nil)
            alertController.addAction(cancelAction)
            self.present(alertController, animated: true, completion: nil)
            return
        }
    }
}
// MARK: - NSNotification
extension MainViewController {
    @objc private func asReaderGUNConnected() {
        print(asReaderGUNConnected)
        DispatchQueue.main.async {
            let asReader = AsReader.init(asReaderGUN: self.appDelegate?.mAsReaderGUN, delegate: self.readerDelegate)
            self.appDelegate?.mReader = asReader
            self.connectMethod()
        }
    }
    @objc private func asReaderGUNDisconnected() {
        print(asreaderGUNDisconnected)
        DispatchQueue.main.async {
            print("Loading Hide...")
            self.appDelegate?.mReader = nil
            self.disableAllUIs()
            AppDelegate.dismissGlobalHUD()
        }
    }
}
